@@ -26,11 +26,15 @@ class NginxConfiguratorTest(util.NginxTest):
 
 
     def setUp(self):
-        super(NginxConfiguratorTest, self).setUp()
+        super().setUp()
 
         self.config = self.get_nginx_configurator(
             self.config_path, self.config_dir, self.work_dir, self.logs_dir)
 
+        patch = mock.patch('certbot_nginx._internal.configurator.display_util.notify')
+        self.mock_notify = patch.start()
+        self.addCleanup(patch.stop)
+
     @mock.patch("certbot_nginx._internal.configurator.util.exe_exists")
     def test_prepare_no_install(self, mock_exe_exists):
         mock_exe_exists.return_value = False
@@ -39,18 +43,19 @@ class NginxConfiguratorTest(util.NginxTest):
 
     def test_prepare(self):
         self.assertEqual((1, 6, 2), self.config.version)
-        self.assertEqual(12, len(self.config.parser.parsed))
+        self.assertEqual(14, len(self.config.parser.parsed))
 
     @mock.patch("certbot_nginx._internal.configurator.util.exe_exists")
-    @mock.patch("certbot_nginx._internal.configurator.subprocess.Popen")
-    def test_prepare_initializes_version(self, mock_popen, mock_exe_exists):
-        mock_popen().communicate.return_value = (
-            "", "\n".join(["nginx version: nginx/1.6.2",
+    @mock.patch("certbot_nginx._internal.configurator.subprocess.run")
+    def test_prepare_initializes_version(self, mock_run, mock_exe_exists):
+        mock_run.return_value.stdout = ""
+        mock_run.return_value.stderr = "\n".join(
+                          ["nginx version: nginx/1.6.2",
                            "built by clang 6.0 (clang-600.0.56)"
                            " (based on LLVM 3.5svn)",
                            "TLS SNI support enabled",
                            "configure arguments: --prefix=/usr/local/Cellar/"
-                           "nginx/1.6.2 --with-http_ssl_module"]))
+                           "nginx/1.6.2 --with-http_ssl_module"])
 
         mock_exe_exists.return_value = True
 
@@ -89,7 +94,7 @@ class NginxConfiguratorTest(util.NginxTest):
             "155.225.50.69.nephoscale.net", "www.example.org", "another.alias",
              "migration.com", "summer.com", "geese.com", "sslon.com",
              "globalssl.com", "globalsslsetssl.com", "ipv6.com", "ipv6ssl.com",
-             "headers.com", "example.net"})
+             "headers.com", "example.net", "ssl.both.com"})
 
     def test_supported_enhancements(self):
         self.assertEqual(['redirect', 'ensure-http-header', 'staple-ocsp'],
@@ -347,141 +352,149 @@ class NginxConfiguratorTest(util.NginxTest):
         self.assertEqual(mock_revert.call_count, 1)
         self.assertEqual(mock_restart.call_count, 2)
 
-    @mock.patch("certbot_nginx._internal.configurator.subprocess.Popen")
-    def test_get_version(self, mock_popen):
-        mock_popen().communicate.return_value = (
-            "", "\n".join(["nginx version: nginx/1.4.2",
+    @mock.patch("certbot_nginx._internal.configurator.subprocess.run")
+    def test_get_version(self, mock_run):
+        mock_run.return_value.stdout = ""
+        mock_run.return_value.stderr = "\n".join(
+                          ["nginx version: nginx/1.4.2",
                            "built by clang 6.0 (clang-600.0.56)"
                            " (based on LLVM 3.5svn)",
                            "TLS SNI support enabled",
                            "configure arguments: --prefix=/usr/local/Cellar/"
-                           "nginx/1.6.2 --with-http_ssl_module"]))
+                           "nginx/1.6.2 --with-http_ssl_module"])
         self.assertEqual(self.config.get_version(), (1, 4, 2))
 
-        mock_popen().communicate.return_value = (
-            "", "\n".join(["nginx version: nginx/0.9",
+        mock_run.return_value.stdout = ""
+        mock_run.return_value.stderr = "\n".join(
+                          ["nginx version: nginx/0.9",
                            "built by clang 6.0 (clang-600.0.56)"
                            " (based on LLVM 3.5svn)",
                            "TLS SNI support enabled",
-                           "configure arguments: --with-http_ssl_module"]))
+                           "configure arguments: --with-http_ssl_module"])
         self.assertEqual(self.config.get_version(), (0, 9))
 
-        mock_popen().communicate.return_value = (
-            "", "\n".join(["blah 0.0.1",
+        mock_run.return_value.stdout = ""
+        mock_run.return_value.stderr = "\n".join(
+                          ["blah 0.0.1",
                            "built by clang 6.0 (clang-600.0.56)"
                            " (based on LLVM 3.5svn)",
                            "TLS SNI support enabled",
-                           "configure arguments: --with-http_ssl_module"]))
+                           "configure arguments: --with-http_ssl_module"])
         self.assertRaises(errors.PluginError, self.config.get_version)
 
-        mock_popen().communicate.return_value = (
-            "", "\n".join(["nginx version: nginx/1.4.2",
-                           "TLS SNI support enabled"]))
+        mock_run.return_value.stdout = ""
+        mock_run.return_value.stderr = "\n".join(
+                          ["nginx version: nginx/1.4.2",
+                           "TLS SNI support enabled"])
         self.assertRaises(errors.PluginError, self.config.get_version)
 
-        mock_popen().communicate.return_value = (
-            "", "\n".join(["nginx version: nginx/1.4.2",
+        mock_run.return_value.stdout = ""
+        mock_run.return_value.stderr = "\n".join(
+                          ["nginx version: nginx/1.4.2",
                            "built by clang 6.0 (clang-600.0.56)"
                            " (based on LLVM 3.5svn)",
-                           "configure arguments: --with-http_ssl_module"]))
+                           "configure arguments: --with-http_ssl_module"])
         self.assertRaises(errors.PluginError, self.config.get_version)
 
-        mock_popen().communicate.return_value = (
-            "", "\n".join(["nginx version: nginx/0.8.1",
+        mock_run.return_value.stdout = ""
+        mock_run.return_value.stderr = "\n".join(
+                          ["nginx version: nginx/0.8.1",
                            "built by clang 6.0 (clang-600.0.56)"
                            " (based on LLVM 3.5svn)",
                            "TLS SNI support enabled",
-                           "configure arguments: --with-http_ssl_module"]))
+                           "configure arguments: --with-http_ssl_module"])
         self.assertRaises(errors.NotSupportedError, self.config.get_version)
 
-        mock_popen.side_effect = OSError("Can't find program")
+        mock_run.side_effect = OSError("Can't find program")
         self.assertRaises(errors.PluginError, self.config.get_version)
 
-    @mock.patch("certbot_nginx._internal.configurator.subprocess.Popen")
-    def test_get_openssl_version(self, mock_popen):
+    @mock.patch("certbot_nginx._internal.configurator.subprocess.run")
+    def test_get_openssl_version(self, mock_run):
         # pylint: disable=protected-access
-        mock_popen().communicate.return_value = (
-            "", """
+        mock_run.return_value.stdout = ""
+        mock_run.return_value.stderr = """
                 nginx version: nginx/1.15.5
                 built by gcc 5.4.0 20160609 (Ubuntu 5.4.0-6ubuntu1~16.04.9)
                 built with OpenSSL 1.0.2g  1 Mar 2016
                 TLS SNI support enabled
                 configure arguments:
-            """)
+            """
         self.assertEqual(self.config._get_openssl_version(), "1.0.2g")
 
-        mock_popen().communicate.return_value = (
-            "", """
+        mock_run.return_value.stdout = ""
+        mock_run.return_value.stderr = """
                 nginx version: nginx/1.15.5
                 built by gcc 5.4.0 20160609 (Ubuntu 5.4.0-6ubuntu1~16.04.9)
                 built with OpenSSL 1.0.2-beta1  1 Mar 2016
                 TLS SNI support enabled
                 configure arguments:
-            """)
+            """
         self.assertEqual(self.config._get_openssl_version(), "1.0.2-beta1")
 
-        mock_popen().communicate.return_value = (
-            "", """
+        mock_run.return_value.stdout = ""
+        mock_run.return_value.stderr = """
                 nginx version: nginx/1.15.5
                 built by gcc 5.4.0 20160609 (Ubuntu 5.4.0-6ubuntu1~16.04.9)
                 built with OpenSSL 1.0.2  1 Mar 2016
                 TLS SNI support enabled
                 configure arguments:
-            """)
+            """
         self.assertEqual(self.config._get_openssl_version(), "1.0.2")
 
-        mock_popen().communicate.return_value = (
-            "", """
+        mock_run.return_value.stdout = ""
+        mock_run.return_value.stderr = """
                 nginx version: nginx/1.15.5
                 built by gcc 5.4.0 20160609 (Ubuntu 5.4.0-6ubuntu1~16.04.9)
                 built with OpenSSL 1.0.2g  1 Mar 2016 (running with OpenSSL 1.0.2a  1 Mar 2016)
                 TLS SNI support enabled
                 configure arguments:
-            """)
+            """
         self.assertEqual(self.config._get_openssl_version(), "1.0.2a")
 
-        mock_popen().communicate.return_value = (
-            "", """
+        mock_run.return_value.stdout = ""
+        mock_run.return_value.stderr = """
                 nginx version: nginx/1.15.5
                 built by gcc 5.4.0 20160609 (Ubuntu 5.4.0-6ubuntu1~16.04.9)
                 built with LibreSSL 2.2.2
                 TLS SNI support enabled
                 configure arguments:
-            """)
+            """
         self.assertEqual(self.config._get_openssl_version(), "")
 
-        mock_popen().communicate.return_value = (
-            "", """
+        mock_run.return_value.stdout = ""
+        mock_run.return_value.stderr = """
                 nginx version: nginx/1.15.5
                 built by gcc 5.4.0 20160609 (Ubuntu 5.4.0-6ubuntu1~16.04.9)
                 TLS SNI support enabled
                 configure arguments:
-            """)
+            """
         self.assertEqual(self.config._get_openssl_version(), "")
 
-    @mock.patch("certbot_nginx._internal.configurator.subprocess.Popen")
+    @mock.patch("certbot_nginx._internal.configurator.subprocess.run")
     @mock.patch("certbot_nginx._internal.configurator.time")
-    def test_nginx_restart(self, mock_time, mock_popen):
-        mocked = mock_popen()
-        mocked.communicate.return_value = ('', '')
+    def test_nginx_restart(self, mock_time, mock_run):
+        mocked = mock_run.return_value
+        mocked.stdout = ''
+        mocked.stderr = ''
         mocked.returncode = 0
         self.config.restart()
-        self.assertEqual(mocked.communicate.call_count, 1)
+        self.assertEqual(mock_run.call_count, 1)
         mock_time.sleep.assert_called_once_with(0.1234)
 
-    @mock.patch("certbot_nginx._internal.configurator.subprocess.Popen")
+    @mock.patch("certbot_nginx._internal.configurator.subprocess.run")
     @mock.patch("certbot_nginx._internal.configurator.logger.debug")
-    def test_nginx_restart_fail(self, mock_log_debug, mock_popen):
-        mocked = mock_popen()
-        mocked.communicate.return_value = ('', '')
+    def test_nginx_restart_fail(self, mock_log_debug, mock_run):
+        mocked = mock_run.return_value
+        mocked.stdout = ''
+        mocked.stderr = ''
         mocked.returncode = 1
         self.assertRaises(errors.MisconfigurationError, self.config.restart)
-        self.assertEqual(mocked.communicate.call_count, 2)
+        self.assertEqual(mock_run.call_count, 2)
         mock_log_debug.assert_called_once_with("nginx reload failed:\n%s", "")
 
-    @mock.patch("certbot_nginx._internal.configurator.subprocess.Popen")
-    def test_no_nginx_start(self, mock_popen):
-        mock_popen.side_effect = OSError("Can't find program")
+    @mock.patch("certbot_nginx._internal.configurator.subprocess.run")
+    def test_no_nginx_start(self, mock_run):
+        mock_run.side_effect = OSError("Can't find program")
         self.assertRaises(errors.MisconfigurationError, self.config.restart)
 
     @mock.patch("certbot.util.run_script")
@@ -935,14 +948,26 @@ class NginxConfiguratorTest(util.NginxTest):
                                                 prefer_ssl=False,
                                                 no_ssl_filter_port='80')
             # Check that the dialog was called with only port 80 vhosts
-            self.assertEqual(len(mock_select_vhs.call_args[0][0]), 6)
+            self.assertEqual(len(mock_select_vhs.call_args[0][0]), 8)
+
+    def test_choose_auth_vhosts(self):
+        """choose_auth_vhosts correctly selects duplicative and HTTP/HTTPS vhosts"""
+        http, https = self.config.choose_auth_vhosts('ssl.both.com')
+        self.assertEqual(len(http), 4)
+        self.assertEqual(len(https), 2)
+        self.assertEqual(http[0].names, {'ssl.both.com'})
+        self.assertEqual(http[1].names, {'ssl.both.com'})
+        self.assertEqual(http[2].names, {'ssl.both.com'})
+        self.assertEqual(http[3].names, {'*.both.com'})
+        self.assertEqual(https[0].names, {'ssl.both.com'})
+        self.assertEqual(https[1].names, {'*.both.com'})
 
 
 class InstallSslOptionsConfTest(util.NginxTest):
     """Test that the options-ssl-nginx.conf file is installed and updated properly."""
 
     def setUp(self):
-        super(InstallSslOptionsConfTest, self).setUp()
+        super().setUp()
 
         self.config = self.get_nginx_configurator(
             self.config_path, self.config_dir, self.work_dir, self.logs_dir)
